==> vice_3.6.1+dfsg.orig.tar.xz: vice-3.6.1+dfsg/src/c64/cart/simonsbasic.c <==
/*
 * simonsbasic.c - Cartridge handling, Simons' Basic cart.
 *
 * Written by
 *  Groepaz <groepaz@gmx.net>
 *
 * This file is part of VICE, the Versatile Commodore Emulator.
 * See README for copyright notice.
 *
 *  This program is free software; you can redistribute it and/or modify
 *  it under the terms of the GNU General Public License as published by
 *  the Free Software Foundation; either version 2 of the License, or
 *  (at your option) any later version.
 *
 *  This program is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *  GNU General Public License for more details.
 *
 *  You should have received a copy of the GNU General Public License
 *  along with this program; if not, write to the Free Software
 *  Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA
 *  02111-1307  USA.
 *
 */

#include "vice.h"

#include <stdio.h>
#include <stdlib.h>
#include <string.h>

#define CARTRIDGE_INCLUDE_SLOTMAIN_API
#include "c64cartsystem.h"
#undef CARTRIDGE_INCLUDE_SLOTMAIN_API
#include "cartio.h"
#include "cartridge.h"
#include "export.h"
#include "monitor.h"
#include "simonsbasic.h"
#include "snapshot.h"
#include "types.h"
#include "util.h"
#include "crt.h"

/*
    Simons' Basic Cartridge

    - 16kb ROM

    - reading io1 switches to 8k game config (bank 0 at $8000)
    - writing io1 switches to 16k game config (bank 0 at $8000, bank 1 at $a000)
*/

static int simon_a000 = 0;

static uint8_t simon_io1_read(uint16_t addr)
{
    cart_config_changed_slotmain(0, 0, CMODE_READ);
    simon_a000 = 0;
    return 0;
}

static uint8_t simon_io1_peek(uint16_t addr)
{
    return 0;
}

static void simon_io1_store(uint16_t addr, uint8_t value)
{
    cart_config_changed_slotmain(1, 1, CMODE_WRITE);
    simon_a000 = 1;
}

static int simon_dump(void)
{
    mon_out("$8000-$9FFF ROM: enabled\n");
    mon_out("$A000-$BFFF ROM: %s\n", (simon_a000) ? "enabled" : "disabled");

    return 0;
}

/* ---------------------------------------------------------------------*/

static io_source_t simon_device = {
    CARTRIDGE_NAME_SIMONS_BASIC, /* name of the device */
    IO_DETACH_CART,              /* use cartridge ID to detach the device when involved in a read-collision */
    IO_DETACH_NO_RESOURCE,       /* does not use a resource for detach */
    0xde00, 0xdeff, 0xff,        /* range for the device, address is ignored, reg:$de00, mirrors:$de01-$deff */
    0,                           /* read is never valid */
    simon_io1_store,             /* store function */
    NULL,                        /* NO poke function */
    simon_io1_read,              /* read function */
    simon_io1_peek,              /* peek function */
    simon_dump,                  /* device state information dump function */
    CARTRIDGE_SIMONS_BASIC,      /* cartridge ID */
    IO_PRIO_NORMAL,              /* normal priority, device read needs to be checked for collisions */
    0                            /* insertion order, gets filled in by the registration function */
};

static io_source_list_t *simon_list_item = NULL;

static const export_resource_t export_res_simon = {
    CARTRIDGE_NAME_SIMONS_BASIC, 1, 1, &simon_device, NULL, CARTRIDGE_SIMONS_BASIC
};

/* ---------------------------------------------------------------------*/

void simon_config_init(void)
{
    cart_config_changed_slotmain(CMODE_16KGAME, CMODE_16KGAME, CMODE_READ);
    simon_a000 = 1;
}

void simon_config_setup(uint8_t *rawcart)
{
    memcpy(roml_banks, rawcart, 0x2000);
    memcpy(romh_banks, &rawcart[0x2000], 0x2000);
    cart_config_changed_slotmain(CMODE_16KGAME, CMODE_16KGAME, CMODE_READ);
    simon_a000 = 1;
}

static int simon_common_attach(void)
{
    if (export_add(&export_res_simon) < 0) {
        return -1;
    }
    simon_list_item = io_source_register(&simon_device);
    return 0;
}

int simon_bin_attach(const char *filename, uint8_t *rawcart)
{
    if (util_file_load(filename, rawcart, 0x4000, UTIL_FILE_LOAD_SKIP_ADDRESS) < 0) {
        return -1;
    }
    return simon_common_attach();
}

int simon_crt_attach(FILE *fd, uint8_t *rawcart)
{
    crt_chip_header_t chip;
    int i;

    for (i = 0; i <= 1; i++) {
        if (crt_read_chip_header(&chip, fd)) {
            return -1;
        }

        if ((chip.start != 0x8000 && chip.start != 0xa000) || chip.size != 0x2000) {
            return -1;
        }

        if (crt_read_chip(rawcart, chip.start - 0x8000, &chip, fd)) {
            return -1;
        }
    }

    return simon_common_attach();
}

void simon_detach(void)
{
    export_remove(&export_res_simon);
    io_source_unregister(simon_list_item);
    simon_list_item = NULL;
}

/* ---------------------------------------------------------------------*/

/* CARTSIMON snapshot module format:

   type  | name        | version | description
   -------------------------------------------
   BYTE  | a000 enable |   0.1   | ROM at $A000 flag
   ARRAY | ROML        |   0.0+  | 8192 BYTES of ROML data
   ARRAY | ROMH        |   0.0+  | 8192 BYTES of ROMH data
 */

static const char snap_module_name[] = "CARTSIMON";
#define SNAP_MAJOR   0
#define SNAP_MINOR   1

int simon_snapshot_write_module(snapshot_t *s)
{
    snapshot_module_t *m;

    m = snapshot_module_create(s, snap_module_name, SNAP_MAJOR, SNAP_MINOR);

    if (m == NULL) {
        return -1;
    }

    if (0
        || SMW_B(m, (uint8_t)simon_a000) < 0
        || SMW_BA(m, roml_banks, 0x2000) < 0
        || SMW_BA(m, romh_banks, 0x2000) < 0) {
        snapshot_module_close(m);
        return -1;
    }

    return snapshot_module_close(m);
}

int simon_snapshot_read_module(snapshot_t *s)
{
    uint8_t vmajor, vminor;
    snapshot_module_t *m;

    m = snapshot_module_open(s, snap_module_name, &vmajor, &vminor);

    if (m == NULL) {
        return -1;
    }

    /* Do not accept versions higher than current */
    if (snapshot_version_is_bigger(vmajor, vminor, SNAP_MAJOR, SNAP_MINOR)) {
        snapshot_set_error(SNAPSHOT_MODULE_HIGHER_VERSION);
        goto fail;
    }

    /* new in 0.1 */
    if (!snapshot_version_is_smaller(vmajor, vminor, 0, 1)) {
        if (SMR_B_INT(m, &simon_a000) < 0) {
            goto fail;
        }
    } else {
        simon_a000 = 0;
    }

    if (0
        || SMR_BA(m, roml_banks, 0x2000) < 0
        || SMR_BA(m, romh_banks, 0x2000) < 0) {
        goto fail;
    }

    snapshot_module_close(m);

    return simon_common_attach();

fail:
    snapshot_module_close(m);
    return -1;
}

==> vice_3.9+dfsg.orig.tar.xz: vice-3.9+dfsg/src/c64/cart/simonsbasic.c <==
/*
 * simonsbasic.c - Cartridge handling, Simons' Basic cart.
 *
 * Written by
 *  Groepaz <groepaz@gmx.net>
 *
 * This file is part of VICE, the Versatile Commodore Emulator.
 * See README for copyright notice.
 *
 *  This program is free software; you can redistribute it and/or modify
 *  it under the terms of the GNU General Public License as published by
 *  the Free Software Foundation; either version 2 of the License, or
 *  (at your option) any later version.
 *
 *  This program is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *  GNU General Public License for more details.
 *
 *  You should have received a copy of the GNU General Public License
 *  along with this program; if not, write to the Free Software
 *  Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA
 *  02111-1307  USA.
 *
 */

#include "vice.h"

#include <stdio.h>
#include <stdlib.h>
#include <string.h>

#define CARTRIDGE_INCLUDE_SLOTMAIN_API
#include "c64cartsystem.h"
#undef CARTRIDGE_INCLUDE_SLOTMAIN_API
#include "cartio.h"
#include "cartridge.h"
#include "export.h"
#include "monitor.h"
#include "simonsbasic.h"
#include "snapshot.h"
#include "types.h"
#include "util.h"
#include "crt.h"

/*
    Simons' Basic Cartridge

    - 16kb ROM

    - reading io1 switches to 8k game config (bank 0 at $8000)
    - writing io1 switches to 16k game config (bank 0 at $8000, bank 1 at $a000)
*/

static int simon_a000 = 0;

static uint8_t simon_io1_read(uint16_t addr)
{
    cart_config_changed_slotmain(0, 0, CMODE_READ);
    simon_a000 = 0;
    return 0;
}

static uint8_t simon_io1_peek(uint16_t addr)
{
    return 0;
}

static void simon_io1_store(uint16_t addr, uint8_t value)
{
    cart_config_changed_slotmain(1, 1, CMODE_WRITE);
    simon_a000 = 1;
}

static int simon_dump(void)
{
    mon_out("$8000-$9FFF ROM: enabled\n");
    mon_out("$A000-$BFFF ROM: %s\n", (simon_a000) ? "enabled" : "disabled");

    return 0;
}

/* ---------------------------------------------------------------------*/

static io_source_t simon_device = {
    CARTRIDGE_NAME_SIMONS_BASIC, /* name of the device */
    IO_DETACH_CART,              /* use cartridge ID to detach the device when involved in a read-collision */
    IO_DETACH_NO_RESOURCE,       /* does not use a resource for detach */
    0xde00, 0xdeff, 0xff,        /* range for the device, address is ignored, reg:$de00, mirrors:$de01-$deff */
    0,                           /* read is never valid */
    simon_io1_store,             /* store function */
    NULL,                        /* NO poke function */
    simon_io1_read,              /* read function */
    simon_io1_peek,              /* peek function */
    simon_dump,                  /* device state information dump function */
    CARTRIDGE_SIMONS_BASIC,      /* cartridge ID */
    IO_PRIO_NORMAL,              /* normal priority, device read needs to be checked for collisions */
    0,                           /* insertion order, gets filled in by the registration function */
    IO_MIRROR_NONE               /* NO mirroring */
};

static io_source_list_t *simon_list_item = NULL;

static const export_resource_t export_res_simon = {
    CARTRIDGE_NAME_SIMONS_BASIC, 1, 1, &simon_device, NULL, CARTRIDGE_SIMONS_BASIC
};

/* ---------------------------------------------------------------------*/

void simon_config_init(void)
{
    cart_config_changed_slotmain(CMODE_16KGAME, CMODE_16KGAME, CMODE_READ);
    simon_a000 = 1;
}

void simon_config_setup(uint8_t *rawcart)
{
    memcpy(roml_banks, rawcart, 0x2000);
    memcpy(romh_banks, &rawcart[0x2000], 0x2000);
    cart_config_changed_slotmain(CMODE_16KGAME, CMODE_16KGAME, CMODE_READ);
    simon_a000 = 1;
}

static int simon_common_attach(void)
{
    if (export_add(&export_res_simon) < 0) {
        return -1;
    }
    simon_list_item = io_source_register(&simon_device);
    return 0;
}

int simon_bin_attach(const char *filename, uint8_t *rawcart)
{
    if (util_file_load(filename, rawcart, 0x4000, UTIL_FILE_LOAD_SKIP_ADDRESS) < 0) {
        return -1;
    }
    return simon_common_attach();
}

int simon_crt_attach(FILE *fd, uint8_t *rawcart)
{
    crt_chip_header_t chip;
    int i;

    for (i = 0; i <= 1; i++) {
        if (crt_read_chip_header(&chip, fd)) {
            return -1;
        }

        if ((chip.start != 0x8000 && chip.start != 0xa000) || chip.size != 0x2000) {
            return -1;
        }

        if (crt_read_chip(rawcart, chip.start - 0x8000, &chip, fd)) {
            return -1;
        }
    }

    return simon_common_attach();
}

void simon_detach(void)
{
    export_remove(&export_res_simon);
    io_source_unregister(simon_list_item);
    simon_list_item = NULL;
}

/* ---------------------------------------------------------------------*/

/* CARTSIMON snapshot module format:

   type  | name        | version | description
   -------------------------------------------
   BYTE  | a000 enable |   0.1   | ROM at $A000 flag
   ARRAY | ROML        |   0.0+  | 8192 BYTES of ROML data
   ARRAY | ROMH        |   0.0+  | 8192 BYTES of ROMH data
 */

static const char snap_module_name[] = "CARTSIMON";
#define SNAP_MAJOR   0
#define SNAP_MINOR   1

int simon_snapshot_write_module(snapshot_t *s)
{
    snapshot_module_t *m;

    m = snapshot_module_create(s, snap_module_name, SNAP_MAJOR, SNAP_MINOR);

    if (m == NULL) {
        return -1;
    }

    if (0
        || SMW_B(m, (uint8_t)simon_a000) < 0
        || SMW_BA(m, roml_banks, 0x2000) < 0
        || SMW_BA(m, romh_banks, 0x2000) < 0) {
        snapshot_module_close(m);
        return -1;
    }

    return snapshot_module_close(m);
}

int simon_snapshot_read_module(snapshot_t *s)
{
    uint8_t vmajor, vminor;
    snapshot_module_t *m;

    m = snapshot_module_open(s, snap_module_name, &vmajor, &vminor);

    if (m == NULL) {
        return -1;
    }

    /* Do not accept versions higher than current */
    if (snapshot_version_is_bigger(vmajor, vminor, SNAP_MAJOR, SNAP_MINOR)) {
        snapshot_set_error(SNAPSHOT_MODULE_HIGHER_VERSION);
        goto fail;
    }

    /* new in 0.1 */
    if (!snapshot_version_is_smaller(vmajor, vminor, 0, 1)) {
        if (SMR_B_INT(m, &simon_a000) < 0) {
            goto fail;
        }
    } else {
        simon_a000 = 0;
    }

    if (0
        || SMR_BA(m, roml_banks, 0x2000) < 0
        || SMR_BA(m, romh_banks, 0x2000) < 0) {
        goto fail;
    }

    snapshot_module_close(m);

    return simon_common_attach();

fail:
    snapshot_module_close(m);
    return -1;
}
